//***************************************************************
// description:
//***************************************************************
// parameter:
//***************************************************************
#pragma
//***************************************************************
// global definition
//***************************************************************
#define LCD_MAT_LIB_C

#include <REG938.H>
#include <lcd_def.h>
#include <intrins.h>
//#include <defines.h>
//#include <globals_V7.h> 
// enthaelt allgemeine Einstellungen zum Projekt
//#include <lcd_def_V7.h>
//#include <cg_ram_lib.h>


//***************************************************************
// Hier muessen die Anpassungen vorgenommen werden, wie das LCD
// an den PCF8574. PCF8575 angeschlossen ist.
//***************************************************************
bdata uchar ucPort;
sbit  sbD0   = ucPort ^ 4;
sbit  LCD_D0 = P2 ^ 4;
sbit  sbD1   = ucPort ^ 5;
sbit  LCD_D1 = P2 ^ 5;
sbit  sbD2   = ucPort ^ 6;
sbit  LCD_D2 = P2 ^ 6;
sbit  sbD3   = ucPort ^ 7;
sbit  LCD_D3 = P2 ^ 7;

sbit sbEN    = ucPort ^ 0;
sbit LCD_EN  = P2 ^ 0;
sbit sbR_W   = ucPort ^ 1;
sbit LCD_R_W = P2 ^ 1;
sbit btRS    = ucPort ^ 2;
sbit LCD_RS  = P2 ^ 2;

#define EN_CLK P2=ucPort; LCD_EN = 1; _nop_(); _nop_(); _nop_(); LCD_EN =0; _nop_(); _nop_(); _nop_();   // @define EN_CLK  | Definitiert eine Schreibsequenz zum HD44780-Controller 

bit btLF = 0;
bit btRS_Old;
bit btID;
uchar ucDD_Adr;   // aktuelle Adresse

//***************************************************************
// Funktionsname v_Wait() 
// Version: 1.0    Datum 
// Uebergabeparameter: uiTime
// Beschreibung: 
// Funktion wait() erzeugt Warteschleifen.                     
//***************************************************************
void v_Wait(uint uiTime) {                  
  uint uiStep;
  if (uiTime == 0) return;
  for (uiStep=0; uiStep < uiTime; uiStep++);
}

//***************************************************************
// Funktionsname v_Busy() 
// Version: 1.0    Datum 
// Uebergabeparameter: keine
// Beschreibung: 
// Funktion v_Busy() wartet ab, bis der HD44780 das busy-Flag zu-
// rueckgesetzt hat.                                          
//***************************************************************
void v_Busy(void) {

  bit btTemp;
  LCD_D0 = LCD_D1 = LCD_D2 = LCD_D3 = SET; // Datenport zum lesen freigeben
  LCD_R_W = LCD_READ;  // Instruktionen sollen vom HD44780 gelesen werdn
  LCD_RS = LCD_INST;
  do 
  {
    LCD_EN = SET; _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); btTemp = LCD_D3; 
	LCD_EN = CLEAR; _nop_(); _nop_(); _nop_(); _nop_(); _nop_();
    LCD_EN = SET; _nop_(); _nop_(); _nop_(); _nop_(); _nop_();
	LCD_EN = CLEAR; _nop_(); _nop_(); _nop_(); _nop_(); _nop_();
  } while (btTemp == SET);  // btLF wird als Zwischenspeicher missbraucht
}

//***************************************************************
// Funktionsname putchar() 
// Version: 1.0    Datum 
// Uebergabeparameter: ucValue
// Beschreibung: 
// Die Funktion printf() verwendet zur Ausgabe von Zeichen     
// die Funktion putchar(). Diese ist als Source beim Kauf vom  
// C51-Compiler im PATH \inc vorhanden. Da die Funktion putchar
// nur einen Parameter uebergeben kann, werden Instruktionen mit
// Ausnahme von \n(Zeilenumbruch) separat durchgefuehrt.        
//***************************************************************
char putchar (uchar ucValue)  {

   if (ucValue == 0x0A && !btCG_Set && (btRS == LCD_DATA)) 
   {   // \n wird von printf in 0x0A 
      btLF = ON; btRS = LCD_INST;                      // umgesetzt
   }
   else 
   {
     if (btLF == ON) 
     {
       btRS = LCD_INST;
       ucDD_Adr = ucValue;
       ucValue = SET_DD_RAM | ucValue;
     }
     else if (btRS == LCD_DATA && btCG_Set == OFF) 
     {
       if (btID) ucDD_Adr++; // Display im Increment-Modus
         else ucDD_Adr--;
     }
     sbR_W = LCD_WRITE;
     ucPort = (ucPort & 0x0F) | (ucValue & 0xF0); // !! Nur gueltig wenn Datenport Px^4-Px^7
     EN_CLK;
     ucPort = (ucPort & 0x0F) | (ucValue << 4);    // !! Nur gueltig wenn Datenport Px^4-Px^7 
     EN_CLK;
     v_Busy();               // BUSY-Abfrage 
     if (btLF == ON) { btLF = OFF; btRS = LCD_DATA; }
   }
   
  return(0);
}


//***************************************************************
// Funktionsname uc_LCDIni() 
// Version: 1.0    Datum 
// Uebergabeparameter: ucLCD_Control
// Beschreibung: 
// Die Funktion uc_LCDIni initialisiert den HD44780-Controller
//***************************************************************
uchar uc_LCDIni(uchar ucLCD_Control){
  uchar ucError = OK;
  uchar ucData, ucStep;

// 1. Ueberpruefung ob das LCD ueber einen 4 bit-datenbus angesprochen
//    wird. In diesem Fall muessen die Initialisierungsdaten bei der
//    Verwendung der unteren 4 Datenbits ins untere Nibble kopiert werden.  
  ucData = INIT_DISP;     

  LCD_EN = LCD_R_W = LCD_RS = CLEAR; // Steuerport zuruecksetzen
  sbR_W = LCD_WRITE;

// 2. Initialisierung des LCD nach Datenblatt (siehe hierzu Seite 95, Teil 2). 
  btRS = LCD_INST;
  for (ucStep = 0; ucStep <3; ucStep++)
  {
    ucPort = ucData;
    EN_CLK;
    v_Wait(500);
  }


//***************************************************************
// Initialisierung ueber Portpins
//***************************************************************
  ucPort = INIT_4BIT;        // !! Nur gueltig wenn Datenport Px^4-Px^7
  EN_CLK;
  v_Wait(50); // Ab jetzt ist BF-Abfrage moeglich

//***************************************************************
// Bei erfolgreicher Initialisierung wird das Display aktiviert
//***************************************************************
  if (ucError == OK) 
  {
    btRS = LCD_INST;
    putchar(FUNCTION_SET | _4BIT | ucLCD_Control); v_Wait(200);
    putchar(FUNCTION_SET | _4BIT | ucLCD_Control);
    putchar(CONTROL | DISPLAY_OFF);
    putchar(CONTROL | DISPLAY_ON | CURSOR_ON | BLINK);
    uc_Clrscr();
    putchar(ENTRY | INCREMENT);
    btID = INCREMENT >>1;
  }
  btRS =LCD_DATA; // Umschalten auf Datenmodus
  return(ucError);
}


//***************************************************************
// Funktionsname uc_ReadData() 
// Version: 1.0    Datum 
// Uebergabeparameter: keine
// Beschreibung: 
// Funktion uc_ReadData() erlaubt Ihnen die Daten vom CG-RAM bzw.
// vom DD-RAM des HD44780 auszulesen. Je nach eingestellter    
// Adresse werden die Daten entweder vom DD-RAM oder CG-RAM    
// ausgelesen.                                                 
//***************************************************************
uchar uc_ReadData(void) {
  bit btRS_Bak;
  uchar erg_uc;

  btRS_Bak = btRS;
  btRS = LCD_DATA;
  LCD_D0 = LCD_D1 = LCD_D2 = LCD_D3 = SET; // Datenport zum lesen freigeben
  LCD_R_W = LCD_READ;  // Instruktionen sollen vom HD44780 gelesen werdn
  LCD_RS = LCD_DATA;
  LCD_EN = SET; erg_uc = P2 & 0x0F; LCD_EN = CLEAR;
  erg_uc = erg_uc << 4;
  LCD_EN = SET; erg_uc = erg_uc | (P2 & 0x0F); LCD_EN = CLEAR;

  btRS = btRS_Bak;
  return(erg_uc);
}

//***************************************************************
// Funktionsname uc_ReadData() 
// Version: 1.0    Datum 
// Uebergabeparameter: ucInst
// Beschreibung: 
// Funktion uc_SetInst() erlaubt Ihnen die Register vom HD44780  
// einzeln zu beschreiben. Dabei wird das Steuerbit RS nach dem
// Schreiben einer Instruction in den alten Zustand zurueckge- 
// setzt.                                                      
//***************************************************************
uchar uc_SetInst(uchar ucInst){
  uchar ucError;
  bit btRS_Bak;

  if ((ucInst & 0xFC) == ENTRY) {
    btID = (ucInst & 0x02) >> 1;
   }
  btRS_Bak = btRS;
  btRS = LCD_INST;
  ucError = putchar(ucInst);
  btRS = btRS_Bak;
  return(ucError);
}


//***************************************************************
// Funktionsname uc_Clrscr() 
// Version: 1.0    Datum 
// Uebergabeparameter: keine
// Beschreibung: 
// Die Funktion uc_Clrscr() loescht den Inhalt vom DD-RAM und setzt
// den Cursor in die linke obere Ecke vom Display.              
//***************************************************************
uchar uc_Clrscr(void) {
  uchar ucError = OK;
  btRS_Old = btRS;
  btRS =LCD_INST;
  ucError = putchar( CLS );                // LCD loeschen 
  if (ucError == OK)
   ucError = putchar( HOME );
  btRS = btRS_Old;
  ucDD_Adr = 0;  // Bei Home wird der interne DD-Zeiger auf 0 
  return(ucError);
}

