#ifndef LCD_DEF_H
  #define LCD_DEF_H

    #define uchar unsigned char
    #define uint  unsigned int
    
    /* Definitionen von Steuersignalen */
    #define INIT_DISP   0x30    // Initsequenz fuer HD44780
    #define INIT_8BIT   0x30
    #define INIT_4BIT   0x20
    
    #define LCD_WRITE 0
    #define LCD_READ  1
    
    #define LCD_INST   0
    #define LCD_DATA   1
    
    #define LEFT   0                /* Richtungsangabe fuer die Funktionen: */
    #define RIGHT  1                /* shift_it(), scroll_it()             */
    
    #define CLS  0x01               /* Display loeschen */
    #define HOME 0x02               /* Cursor an Adresse 0 im Display  */
    #define LCD_BUSY 0x80
    
    #define ENTRY      0x04         /* Setzt die Cursor-Richtung */
    #define INCREMENT  0x02
    #define DISP_SHIFT 0x01
    #define DECREMENT  0x00
    
    
    #define CONTROL      0x08       /* Setzt   Display  ON/ OFF */
    #define DISPLAY_ON   0x04       /*         Cursor   ON/ OFF */
    #define DISPLAY_OFF  0x00       /*         BLINK    ON/ OFF */
    #define CURSOR_ON    0x02
    #define CURSOR_OFF   0x00
    #define BLINK        0x01
    #define NO_BLINK     0x00
    
    #define SHIFT      0x10         /* Bewegt den Cursor und schiebt */
    #define SHI_LEFT   0x00         /* den Displayinhalt ohne den    */
    #define SHI_RIGHT  0x04         /* Inhalt vom DD-RAM zu aendern.  */  
    #define DIS_LEFT   0x08
    #define DIS_RIGHT  0x0C
    
    #define FUNCTION_SET 0x20       /* Setzt die Datenbreite, Zeilenanzahl */
                                    /* und die Darstellung der Zeichen.    */
    #define _8BIT        0x10       /* 8-bit Datenbus                      */
    #define _4BIT        0x00       /* 4-bit Datenbus                      */
    #define _2LINE       0x08       /* zweizeiliges Display                */
    #define _1LINE       0x00       /* einzeiliges Display                 */ 
    #define _10DOTS      0x04       /* 5*10 dots Aufloesung, je nach Displaytyp */
    #define _7DOTS       0x00       /* 5*7  dots */
    
    #define SET_CG_RAM   0x40       /* Setzt den Adresszaehler im CG-RAM   */       
    #define CG_RAM       0x01
    #define SET_DD_RAM   0x80       /* Setzt den Adresszaehler im DD-RAM   */
    #define DD_RAM       0x00
    
    #define ON      1
    #define OFF     0
    #define OK      0   
    #define ERR    -1   
    #define ENABLE  1   
    #define DISABLE 0   
    #define CLEAR   0 
    #define SET     1  

// Definition der Startadresse fuer das erste Zeichen in jeder Zeile
    #define _1_LINE      0x00
    #define _2_LINE      0x40
// bei 4x16 Matrix gelten folgende Anfangsadressen
//    #define _3_LINE      0x10
//    #define _4_LINE      0x50
// bei 4x20 Matrix gelten folgende Anfangsadressen
   #define _3_LINE      0x14
   #define _4_LINE      0x54


extern uchar uc_LCDIni(uchar ucLCD_Control);
extern uchar uc_Clrscr(void);
extern uchar uc_SetInst(uchar ucInst);
extern char  putchar (char c);

extern bit btCG_Set;
extern bit btID;        // include the information from the mode INC/DEC in the HD44780
extern bit btRS;
extern uchar ucDD_Adr;  // pointer for Display DD-RAM


#endif